CREATE OR REPLACE package emp_survey as

   /*
   || Datatype used to hold the responses to the survey
   */
   type response_array is table of varchar2(20) index by binary_integer;
   no_response response_array;

   -- Display the list of available surveys
   procedure display_survey_list;

   -- Display the form so that the user can respond
   procedure answer_survey ( i_survey_id in varchar2 default null);

   -- Save the responses
   procedure process_survey (
      question in response_array default emp_survey.no_response,
      response in response_array default emp_survey.no_response );

   -- Display the results of a survey
   procedure view_results ( i_survey_id in varchar2 default null );

end;


  
/

CREATE OR REPLACE package body emp_survey as

   /*-------------------------------------------------------------------------
   || Display the list of available surveys
   */-------------------------------------------------------------------------
   procedure display_survey_list is
      -- Create cursor of all current surveys
	  cursor survey_cur is
	     select * from survey;
	  survey_rec survey_cur%ROWTYPE;
	  rec_count number default 0;

   begin
      htp.print ('<title>Available Surveys</title>');
	  htp.print ('<body bgcolor=white>');
	  htp.print('<ol>'); -- begin an ordered list
      open survey_cur;
	  loop
	     fetch survey_cur into survey_rec;
		 exit when survey_cur%NOTFOUND;
		 htp.print('<li>');
		 htp.bold( survey_rec.survey_name );
		 htp.print('<br>');
		 htp.print('<i>' || survey_rec.survey_desc || '</i><br>');
		 -- Put an anchor to take the survey
		 htp.anchor(
		    'emp_survey.answer_survey?i_survey_id=' || survey_rec.survey_id,
		    'Take the survey' );
		 -- Put an anchor to view the results
		 htp.anchor(
		    'emp_survey.view_results?i_survey_id=' || survey_rec.survey_id,
		    'View the results' );
		 htp.print('<p></li>');
	  end loop;
	  htp.print('</ol>');
	  close survey_cur;
   end display_survey_list;

   /*-------------------------------------------------------------------------
   || Answer the survey
   */-------------------------------------------------------------------------
   procedure answer_survey ( i_survey_id in varchar2 ) is

      cursor q_cur is
	     select * from questions
		    where survey_id = i_survey_id
			order by question_num;

	  q_rec q_cur%ROWTYPE;

	  stmt varchar2(500);

   begin
      htp.title('Survey');
	  htp.print ('<body bgcolor=white>');
	  htp.print('<form action=emp_survey.process_survey method=POST>');
	  htp.print('<table width=80%>');
	  open q_cur;
	  loop
	     fetch q_cur into q_rec;
		 exit when q_cur%NOTFOUND;
		 htp.print('<tr>');
		 -- Print the question
		 htp.print('<th align=left valign=top>');
		 htp.print( q_rec.question_text);
		 htp.print('</th>');
		 htp.print('<td>'); -- start a new column
		 -- Put the question_id in as a hidden field
		 htp.formHidden(cname=>'question',cvalue=>q_rec.question_id);
		 -- Use owa_util.listprint to build the LOV for the answer
		 stmt := 'select answer_id, answer_text, default_ans_flag ';
		 stmt := stmt || ' from answers where';
	     stmt := stmt || ' question_id = ' ||  q_rec.question_id;
	     stmt := stmt || ' order by answer_order';
		 owa_util.listprint (stmt, 'response', 4, FALSE);
         htp.print('</td>');
		 htp.print('</tr>');
	  end loop; 
	  htp.print('</table>');
	  htp.formSubmit;
	  close q_cur;
   end;

   /*-------------------------------------------------------------------------
   || Save the responses
   */-------------------------------------------------------------------------
   procedure process_survey (
      question in response_array default emp_survey.no_response,
      response in response_array default emp_survey.no_response ) is

	  item_count number default 0;

   begin
      htp.title('Saving Responses');
	  htp.print('<body bgcolor=white>');
	  loop
	     item_count := item_count + 1;
		 exit when (item_count > 100);
		 insert into responses
		    (response_id , question_id, answer_id)
	     values
		    (response_seq.nextval, question(item_count), response(item_count) );
	  end loop;
   -- Exceeding an array bound will trigger an exception
   -- that terminates the loop
   exception
      when NO_DATA_FOUND then
         htp.print ('<h1>Values Saved</h1><hr>');
		 display_survey_list;
   end;

   /*-------------------------------------------------------------------------
   || Print a basic report about the output
   */-------------------------------------------------------------------------
   procedure print_answers (i_question_id in varchar2) is
      cursor ans_cur is
	     select * from answers where question_id = i_question_id
		    order by answer_order;
	   ans_rec ans_cur%ROWTYPE;
	   total_responses number;
       num_responses number;
   begin
      -- Fecth the total number of responses
      select count(*) into total_responses from responses where
	     question_id = i_question_id;
      htp.print('<table width=50%>');
      open ans_cur;
	  loop
	     fetch ans_cur into ans_rec;
		 exit when ans_cur%NOTFOUND;
		 htp.print('<tr>');
		 -- Print question text
		 htp.print('<td nowrap>' || ans_rec.answer_text || '</td>');
		 -- Fetch and print number of responses
		 select count(*) into num_responses from responses where
		    answer_id = ans_rec.answer_id;
		 if total_responses > 0 then
   		    htp.print('<td align=right>' || round(num_responses/total_responses*100,2) || '%</td>');
		 else
   		    htp.print('<td align=right>***</td>');
         end if;
		 htp.print('</tr>');
      end loop;
	  htp.print('</table>');
	  close ans_cur;
   end;

   procedure view_results ( i_survey_id in varchar2 ) is
      cursor q_cur is
	     select * from questions where survey_id = i_survey_id
		    order by question_num;
	  q_rec q_cur%ROWTYPE;
   begin
      htp.title('Survey Responses');
	  htp.print('<body bgcolor=white>');
	  htp.print('<ol>');
	  open q_cur;
	  loop
	     fetch q_cur into q_rec;
		 exit when q_cur%NOTFOUND;
		 htp.print('<li>');
		 htp.print('<b>' || q_rec.question_text || '</b>');
		 htp.print('<br>');
		 print_answers(q_rec.question_id);
		 htp.print('</td>');
      end loop;
	  htp.print('</ol>');
	  close q_cur;
   end;


end;


  
/

